import { ReduxActionContextDecorator, ReduxActionDecorator, ReduxActionDispatcher, ReduxReducerDecorator, ReduxStateDecorator } from '@harmowatch/redux-decorators';
import 'rxjs/add/operator/distinctUntilChanged';
import { Inject, Injectable, InjectionToken, Injector, NgModule, Optional, Pipe, isDevMode } from '@angular/core';
import 'rxjs/add/operator/map';
import { ReplaySubject as ReplaySubject$1 } from 'rxjs/ReplaySubject';
import 'rxjs/add/operator/toPromise';
import { AsyncSubject as AsyncSubject$1 } from 'rxjs/AsyncSubject';
import { Observable as Observable$1 } from 'rxjs/Observable';
import { CommonModule } from '@angular/common';
import 'rxjs/add/operator/takeWhile';
import { BehaviorSubject as BehaviorSubject$1 } from 'rxjs/BehaviorSubject';
import { createStore } from 'redux';

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
class StateDefToken extends InjectionToken {
    constructor() {
        super('StateDefToken');
    }
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
class ReduxSelectPipe {
    /**
     * @param {?=} stateDefs
     * @param {?=} injector
     */
    constructor(stateDefs = [], injector) {
        this.provider = /** @type {?} */ (injector.get(stateDefs[0].provider));
    }
    /**
     * @param {?} selector
     * @return {?}
     */
    transform(selector) {
        return this.provider.select(selector);
    }
}
ReduxSelectPipe.decorators = [
    { type: Pipe, args: [{ name: 'reduxSelect' },] },
];
/** @nocollapse */
ReduxSelectPipe.ctorParameters = () => [
    { type: Array, decorators: [{ type: Inject, args: [StateDefToken,] },] },
    { type: Injector, },
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */

/**
 * @record
 */

class Registry {
    /**
     * @return {?}
     */
    static reset() {
        Registry._store = new AsyncSubject$1();
        Registry._reducers = [];
    }
    /**
     * @param {?} store
     * @return {?}
     */
    static registerStore(store) {
        Registry.reset();
        Registry._store.next(store);
        Registry._store.complete();
        ReduxActionDispatcher.dispatchedActions.subscribe(action => {
            store.dispatch(/** @type {?} */ (action));
        });
    }
    /**
     * @param {?} state
     * @return {?}
     */
    static registerState(state) {
        Registry.getStore().then((store) => {
            const /** @type {?} */ stateConfig = ReduxStateDecorator.get(state.constructor);
            const /** @type {?} */ initState = state.getInitialState();
            let /** @type {?} */ initStateToResolve = initState;
            if (initState instanceof Observable$1) {
                initStateToResolve = initState.toPromise();
            }
            Promise.resolve(initStateToResolve).then(initialState => {
                store.dispatch({
                    payload: {
                        initialValue: initialState,
                        name: stateConfig.name,
                    },
                    type: Registry.ACTION_REGISTER_STATE,
                });
            });
        });
    }
    /**
     * @return {?}
     */
    static getStore() {
        return new Promise(Registry._store.subscribe.bind(Registry._store));
    }
}
Registry.ACTION_REGISTER_STATE = `@harmowatch/ngx-redux-core/registerState`;
Registry._store = new AsyncSubject$1();
Registry._reducers = [];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
class ReduxSelector extends ReplaySubject$1 {
    /**
     * @param {?=} selector
     * @param {?=} stateProvider
     */
    constructor(selector = '/', stateProvider) {
        if (!selector.startsWith(ReduxSelector.DELIMITER) && !stateProvider) {
            throw new Error('You need to provide a state provider, if you use relative selectors');
        }
        super(1);
        Registry.getStore().then(store => {
            const /** @type {?} */ next = () => {
                this.next(ReduxSelector.getValueByState(store.getState(), selector, stateProvider));
            };
            store.subscribe(() => next());
            next(); // we need to trigger a initial value, otherwise we've to wait until the first state change
        });
    }
    /**
     * @param {?} selector
     * @param {?=} stateProvider
     * @return {?}
     */
    static normalize(selector, stateProvider) {
        if (!selector.startsWith(ReduxSelector.DELIMITER)) {
            return `/${ReduxStateDecorator.get(stateProvider).name}/${selector}`;
        }
        return selector;
    }
    /**
     * @template S
     * @param {?} state
     * @param {?} selector
     * @param {?=} stateProvider
     * @return {?}
     */
    static getValueByState(state, selector, stateProvider) {
        /* save the return value in a constant to prevent
             * "Metadata collected contains an error that will be reported at runtime: Lambda not supported."
             * error
             */
        const /** @type {?} */ value = ReduxSelector.normalize(selector, stateProvider).split(ReduxSelector.DELIMITER)
            .filter(propertyKey => propertyKey !== '')
            .reduce((previousValue, propertyKey) => {
            if (!previousValue || !previousValue.hasOwnProperty(propertyKey)) {
                return null;
            }
            return previousValue[propertyKey];
        }, /** @type {?} */ (state));
        return value;
    }
}
ReduxSelector.DELIMITER = '/';

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
/**
 * @abstract
 */
class ReduxStateProvider {
    /**
     * @param {?=} stateDefs
     */
    constructor(stateDefs = []) {
        this.selectorCache = {};
        const { name = null } = ReduxStateDecorator.get(this.constructor) || {};
        if (!name) {
            throw new Error('Unable to resolve state name! Make sure you\'ve decorated the provider by "@ReduxState"!');
        }
        this.name = ReduxStateDecorator.get(this.constructor).name;
        this.stateDef = stateDefs.find(def => ReduxStateDecorator.get(def.provider).name === name);
        if (!this.stateDef) {
            throw new Error('Unable to resolve state definition! Make sure you\'ve registered the provider to ReduxModule!');
        }
        this.reducerMethodsByType = (this.stateDef.reducers || [])
            .map(clazz => this.getReducerMethods(new clazz()))
            .reduce((all, curr) => [].concat(curr, all), []) // [].concat keeps the order, all.concat(curr) destroys the order
            .reduce((methodsByType, reducer) => {
            const /** @type {?} */ type = getActionType(reducer.type);
            return Object.assign({}, methodsByType, { [type]: [reducer.method].concat(methodsByType[type] || []) });
        }, {});
        ReduxStateProvider.instancesByName[this.name] = this;
    }
    /**
     * @return {?}
     */
    getInitialState() {
        throw new Error('Method "getInitialState" not implemented.');
    }
    /**
     * @template T
     * @param {?=} selector
     * @return {?}
     */
    select(selector = '') {
        const /** @type {?} */ stateType = /** @type {?} */ (this.constructor);
        selector = ReduxSelector.normalize(selector, stateType);
        if (!this.selectorCache[selector]) {
            this.selectorCache[selector] = new ReduxSelector(selector, stateType).distinctUntilChanged();
        }
        return /** @type {?} */ (this.selectorCache[selector]);
    }
    /**
     * @template P
     * @param {?} state
     * @param {?} action
     * @return {?}
     */
    reduce(state, action) {
        const /** @type {?} */ reducerMethods = this.reducerMethodsByType[action.type] || [];
        return reducerMethods.reduce((stateToReduce, method) => method(stateToReduce, action), state);
    }
    /**
     * @param {?} reducerClassInstance
     * @return {?}
     */
    getReducerMethods(reducerClassInstance) {
        return Object.values(Object.getPrototypeOf(reducerClassInstance))
            .map(method => {
            return {
                method: (/** @type {?} */ (method)).bind(reducerClassInstance),
                type: ReduxReducerDecorator.get(method),
            };
        })
            .filter(type => type != null)
            .reduce((all, curr) => all.concat([].concat(curr.type).map(type => (Object.assign({}, curr, { type })))), []);
    }
}
ReduxStateProvider.instancesByName = {};
/** @nocollapse */
ReduxStateProvider.ctorParameters = () => [
    { type: Array, decorators: [{ type: Inject, args: [StateDefToken,] },] },
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
/**
 * @template S
 * @param {?} expression
 * @param {?=} context
 * @return {?}
 */
function ReduxSelect(expression, context) {
    return (target, propertyKey) => {
        const /** @type {?} */ originalOnInit = target['ngOnInit'];
        Object.defineProperty(target, 'ngOnInit', {
            value: function (injector) {
                const /** @type {?} */ stateName = ReduxStateDecorator.get(context).name;
                Object.defineProperty(target, propertyKey, {
                    value: ReduxStateProvider.instancesByName[stateName].select(expression)
                });
                if (originalOnInit) {
                    originalOnInit.apply(target, arguments);
                }
            }
        });
    };
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
const ReduxState = ReduxStateDecorator.forClass;

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
class ReduxStore extends InjectionToken {
    constructor() {
        super('ReduxStore');
    }
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
class ReduxCommonModule {
}
ReduxCommonModule.decorators = [
    { type: NgModule, args: [{
                declarations: [
                    ReduxSelectPipe,
                ],
                exports: [
                    ReduxSelectPipe,
                ],
            },] },
];
/** @nocollapse */
ReduxCommonModule.ctorParameters = () => [];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
class ReduxTestingStore {
    constructor() {
        this.state = new BehaviorSubject$1(null);
    }
    /**
     * @template S
     * @param {?} state
     * @param {?} value
     * @return {?}
     */
    setState(state, value) {
        const { name } = ReduxStateDecorator.get(state);
        const /** @type {?} */ nextState = Object.assign({}, this.state.getValue(), {
            [name]: value,
        });
        this.state.next(nextState);
        return this.state
            .takeWhile(currentState => currentState !== nextState)
            .toPromise().then(() => this.state.getValue());
    }
    /**
     * @return {?}
     */
    getState() {
        return this.state.getValue();
    }
    /**
     * @param {?} listener
     * @return {?}
     */
    subscribe(listener) {
        return this.state.subscribe.call(this.state, listener);
    }
    /**
     * @return {?}
     */
    replaceReducer() {
    }
    /**
     * @template T
     * @param {?} action
     * @return {?}
     */
    dispatch(action) {
        return action;
    }
}
ReduxTestingStore.decorators = [
    { type: Injectable },
];
/** @nocollapse */
ReduxTestingStore.ctorParameters = () => [];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
class ReduxTestingModule {
    /**
     * @param {?=} store
     */
    constructor(store = null) {
        if (store) {
            Registry.reset();
            Registry.registerStore(store);
        }
    }
    /**
     * @param {?=} config
     * @return {?}
     */
    static forRoot(config = {}) {
        return {
            ngModule: ReduxTestingModule,
            providers: [
                { provide: ReduxTestingStore, useFactory: config.storeFactory || ReduxTestingModule.defaultStoreFactory },
                { provide: StateDefToken, useValue: config.state || null, multi: true },
            ],
        };
    }
    /**
     * @return {?}
     */
    static defaultStoreFactory() {
        return new ReduxTestingStore();
    }
}
ReduxTestingModule.decorators = [
    { type: NgModule, args: [{
                exports: [
                    ReduxCommonModule,
                ],
                imports: [
                    CommonModule,
                ],
                providers: [
                    ReduxTestingStore,
                ],
            },] },
];
/** @nocollapse */
ReduxTestingModule.ctorParameters = () => [
    { type: undefined, decorators: [{ type: Optional }, { type: Inject, args: [ReduxTestingStore,] },] },
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
class ReducerProvider {
    constructor() {
        this.stateProviders = {};
    }
    /**
     * @param {?} provider
     * @return {?}
     */
    addStateProvider(provider) {
        if (!this.stateProviders[provider.name]) {
            Registry.registerState(provider);
            this.stateProviders[provider.name] = provider;
        }
    }
    /**
     * @param {?} rootState
     * @param {?} action
     * @return {?}
     */
    reduce(rootState, action) {
        if (action.type === Registry.ACTION_REGISTER_STATE) {
            const /** @type {?} */ regAction = (/** @type {?} */ ((action)));
            return Object.assign({}, rootState, { [regAction.payload.name]: regAction.payload.initialValue });
        }
        return Object.values(this.stateProviders).reduce((stateToReduce, provider) => {
            return Object.assign({}, stateToReduce, {
                [provider.name]: provider.reduce(stateToReduce[provider.name], action),
            });
        }, rootState);
    }
}
ReducerProvider.decorators = [
    { type: Injectable },
];
/** @nocollapse */
ReducerProvider.ctorParameters = () => [];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
class ReduxRootModule {
    /**
     * @param {?=} store
     * @param {?=} reducer
     * @param {?=} injector
     * @param {?=} stateDefs
     */
    constructor(store = null, reducer, injector, stateDefs = []) {
        this.reducer = reducer;
        this.injector = injector;
        Registry.registerStore(store);
        stateDefs
            .filter(def => def && def.provider)
            .map(def => this.injector.get(def.provider))
            .forEach(provider => this.reducer.addStateProvider(provider));
    }
}
ReduxRootModule.decorators = [
    { type: NgModule, args: [{
                exports: [
                    ReduxCommonModule,
                ],
                imports: [
                    CommonModule,
                ],
            },] },
];
/** @nocollapse */
ReduxRootModule.ctorParameters = () => [
    { type: undefined, decorators: [{ type: Inject, args: [ReduxStore,] },] },
    { type: ReducerProvider, },
    { type: Injector, },
    { type: Array, decorators: [{ type: Optional }, { type: Inject, args: [StateDefToken,] },] },
];
class ReduxModule {
    /**
     * @param {?=} stateDefs
     * @param {?=} reducer
     * @param {?=} injector
     */
    constructor(stateDefs = [], reducer, injector) {
        this.reducer = reducer;
        this.injector = injector;
        stateDefs
            .filter(def => def && def.provider)
            .map(def => this.injector.get(def.provider))
            .forEach(provider => this.reducer.addStateProvider(provider));
    }
    /**
     * @param {?=} config
     * @return {?}
     */
    static forChild(config = {}) {
        return {
            ngModule: ReduxModule,
            providers: [
                { provide: StateDefToken, useValue: config.state || null, multi: true },
            ],
        };
    }
    /**
     * @param {?=} config
     * @return {?}
     */
    static forRoot(config = {}) {
        return {
            ngModule: ReduxRootModule,
            providers: [
                ReducerProvider,
                {
                    provide: ReduxStore,
                    useFactory: config.storeFactory || ReduxModule.defaultStoreFactory,
                    deps: [ReducerProvider]
                },
                { provide: StateDefToken, useValue: config.state || null, multi: true },
            ],
        };
    }
    /**
     * @param {?} reducer
     * @return {?}
     */
    static defaultStoreFactory(reducer) {
        return createStore(reducer.reduce.bind(reducer), {}, ReduxModule.defaultEnhancerFactory());
    }
    /**
     * @return {?}
     */
    static defaultEnhancerFactory() {
        if (console && window['__REDUX_DEVTOOLS_EXTENSION__'] && isDevMode()) {
            return window['__REDUX_DEVTOOLS_EXTENSION__']();
        }
        return ReduxModule.noopEnhancer;
    }
    /**
     * @param {?} next
     * @return {?}
     */
    static noopEnhancer(next) {
        return next;
    }
}
ReduxModule.decorators = [
    { type: NgModule, args: [{
                exports: [
                    ReduxCommonModule,
                ],
                imports: [
                    CommonModule,
                ],
            },] },
];
/** @nocollapse */
ReduxModule.ctorParameters = () => [
    { type: Array, decorators: [{ type: Optional }, { type: Inject, args: [StateDefToken,] },] },
    { type: ReducerProvider, },
    { type: Injector, },
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
const ReduxAction = ReduxActionDecorator.forMethod;
const ReduxActionContext = ReduxActionContextDecorator.forClass;
const getActionType = ReduxActionDispatcher.getType;
const ReduxReducer = ReduxReducerDecorator.forMethod;
/**
 * @record
 */

/**
 * interface doesn't work \@see https://github.com/angular/angular-cli/issues/2034#issuecomment-302666897 :/
 * @deprecated Please use ActionWithPayload instead
 */
class ActionInterface {
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
/**
 * Generated bundle index. Do not edit.
 */

export { ReduxAction, ReduxActionContext, getActionType, ReduxReducer, ActionInterface, ReduxSelect, ReduxSelectPipe, ReduxState, ReduxStore, ReduxTestingModule, ReduxTestingStore, ReduxSelector, ReduxStateProvider, ReduxRootModule, ReduxModule, ReduxCommonModule as ɵb, ReducerProvider as ɵc, StateDefToken as ɵa };
//# sourceMappingURL=harmowatch-ngx-redux-core.js.map
