var __extends = (this && this.__extends) || (function () {
    var extendStatics = Object.setPrototypeOf ||
        ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
        function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
import { ReduxActionContextDecorator, ReduxActionDecorator, ReduxActionDispatcher, ReduxReducerDecorator, ReduxStateDecorator } from '@harmowatch/redux-decorators';
import 'rxjs/add/operator/distinctUntilChanged';
import { Inject, Injectable, InjectionToken, Injector, NgModule, Optional, Pipe, isDevMode } from '@angular/core';
import 'rxjs/add/operator/map';
import { ReplaySubject as ReplaySubject$1 } from 'rxjs/ReplaySubject';
import 'rxjs/add/operator/toPromise';
import { AsyncSubject as AsyncSubject$1 } from 'rxjs/AsyncSubject';
import { Observable as Observable$1 } from 'rxjs/Observable';
import { CommonModule } from '@angular/common';
import 'rxjs/add/operator/takeWhile';
import { BehaviorSubject as BehaviorSubject$1 } from 'rxjs/BehaviorSubject';
import { createStore } from 'redux';
/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
var StateDefToken = /** @class */ (function (_super) {
    __extends(StateDefToken, _super);
    function StateDefToken() {
        return _super.call(this, 'StateDefToken') || this;
    }
    return StateDefToken;
}(InjectionToken));
/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
var ReduxSelectPipe = /** @class */ (function () {
    /**
     * @param {?=} stateDefs
     * @param {?=} injector
     */
    function ReduxSelectPipe(stateDefs, injector) {
        if (stateDefs === void 0) { stateDefs = []; }
        this.provider = /** @type {?} */ (injector.get(stateDefs[0].provider));
    }
    /**
     * @param {?} selector
     * @return {?}
     */
    ReduxSelectPipe.prototype.transform = function (selector) {
        return this.provider.select(selector);
    };
    return ReduxSelectPipe;
}());
ReduxSelectPipe.decorators = [
    { type: Pipe, args: [{ name: 'reduxSelect' },] },
];
/** @nocollapse */
ReduxSelectPipe.ctorParameters = function () { return [
    { type: Array, decorators: [{ type: Inject, args: [StateDefToken,] },] },
    { type: Injector, },
]; };
/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
/**
 * @record
 */
var Registry = /** @class */ (function () {
    function Registry() {
    }
    /**
     * @return {?}
     */
    Registry.reset = function () {
        Registry._store = new AsyncSubject$1();
        Registry._reducers = [];
    };
    /**
     * @param {?} store
     * @return {?}
     */
    Registry.registerStore = function (store) {
        Registry.reset();
        Registry._store.next(store);
        Registry._store.complete();
        ReduxActionDispatcher.dispatchedActions.subscribe(function (action) {
            store.dispatch(/** @type {?} */ (action));
        });
    };
    /**
     * @param {?} state
     * @return {?}
     */
    Registry.registerState = function (state) {
        Registry.getStore().then(function (store) {
            var /** @type {?} */ stateConfig = ReduxStateDecorator.get(state.constructor);
            var /** @type {?} */ initState = state.getInitialState();
            var /** @type {?} */ initStateToResolve = initState;
            if (initState instanceof Observable$1) {
                initStateToResolve = initState.toPromise();
            }
            Promise.resolve(initStateToResolve).then(function (initialState) {
                store.dispatch({
                    payload: {
                        initialValue: initialState,
                        name: stateConfig.name,
                    },
                    type: Registry.ACTION_REGISTER_STATE,
                });
            });
        });
    };
    /**
     * @return {?}
     */
    Registry.getStore = function () {
        return new Promise(Registry._store.subscribe.bind(Registry._store));
    };
    return Registry;
}());
Registry.ACTION_REGISTER_STATE = "@harmowatch/ngx-redux-core/registerState";
Registry._store = new AsyncSubject$1();
Registry._reducers = [];
/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
var ReduxSelector = /** @class */ (function (_super) {
    __extends(ReduxSelector, _super);
    /**
     * @param {?=} selector
     * @param {?=} stateProvider
     */
    function ReduxSelector(selector, stateProvider) {
        if (selector === void 0) { selector = '/'; }
        var _this = this;
        if (!selector.startsWith(ReduxSelector.DELIMITER) && !stateProvider) {
            throw new Error('You need to provide a state provider, if you use relative selectors');
        }
        _this = _super.call(this, 1) || this;
        Registry.getStore().then(function (store) {
            var /** @type {?} */ next = function () {
                _this.next(ReduxSelector.getValueByState(store.getState(), selector, stateProvider));
            };
            store.subscribe(function () { return next(); });
            next(); // we need to trigger a initial value, otherwise we've to wait until the first state change
        });
        return _this;
    }
    /**
     * @param {?} selector
     * @param {?=} stateProvider
     * @return {?}
     */
    ReduxSelector.normalize = function (selector, stateProvider) {
        if (!selector.startsWith(ReduxSelector.DELIMITER)) {
            return "/" + ReduxStateDecorator.get(stateProvider).name + "/" + selector;
        }
        return selector;
    };
    /**
     * @template S
     * @param {?} state
     * @param {?} selector
     * @param {?=} stateProvider
     * @return {?}
     */
    ReduxSelector.getValueByState = function (state, selector, stateProvider) {
        /* save the return value in a constant to prevent
             * "Metadata collected contains an error that will be reported at runtime: Lambda not supported."
             * error
             */
        var /** @type {?} */ value = ReduxSelector.normalize(selector, stateProvider).split(ReduxSelector.DELIMITER)
            .filter(function (propertyKey) { return propertyKey !== ''; })
            .reduce(function (previousValue, propertyKey) {
            if (!previousValue || !previousValue.hasOwnProperty(propertyKey)) {
                return null;
            }
            return previousValue[propertyKey];
        }, /** @type {?} */ (state));
        return value;
    };
    return ReduxSelector;
}(ReplaySubject$1));
ReduxSelector.DELIMITER = '/';
/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
/**
 * @abstract
 */
var ReduxStateProvider = /** @class */ (function () {
    /**
     * @param {?=} stateDefs
     */
    function ReduxStateProvider(stateDefs) {
        if (stateDefs === void 0) { stateDefs = []; }
        var _this = this;
        this.selectorCache = {};
        var _a = (ReduxStateDecorator.get(this.constructor) || {}).name, name = _a === void 0 ? null : _a;
        if (!name) {
            throw new Error('Unable to resolve state name! Make sure you\'ve decorated the provider by "@ReduxState"!');
        }
        this.name = ReduxStateDecorator.get(this.constructor).name;
        this.stateDef = stateDefs.find(function (def) { return ReduxStateDecorator.get(def.provider).name === name; });
        if (!this.stateDef) {
            throw new Error('Unable to resolve state definition! Make sure you\'ve registered the provider to ReduxModule!');
        }
        this.reducerMethodsByType = (this.stateDef.reducers || [])
            .map(function (clazz) { return _this.getReducerMethods(new clazz()); })
            .reduce(function (all, curr) { return [].concat(curr, all); }, []) // [].concat keeps the order, all.concat(curr) destroys the order
            .reduce(function (methodsByType, reducer) {
            var /** @type {?} */ type = getActionType(reducer.type);
            return Object.assign({}, methodsByType, (_a = {}, _a[type] = [reducer.method].concat(methodsByType[type] || []), _a));
            var _a;
        }, {});
        ReduxStateProvider.instancesByName[this.name] = this;
    }
    /**
     * @return {?}
     */
    ReduxStateProvider.prototype.getInitialState = function () {
        throw new Error('Method "getInitialState" not implemented.');
    };
    /**
     * @template T
     * @param {?=} selector
     * @return {?}
     */
    ReduxStateProvider.prototype.select = function (selector) {
        if (selector === void 0) { selector = ''; }
        var /** @type {?} */ stateType = (this.constructor);
        selector = ReduxSelector.normalize(selector, stateType);
        if (!this.selectorCache[selector]) {
            this.selectorCache[selector] = new ReduxSelector(selector, stateType).distinctUntilChanged();
        }
        return /** @type {?} */ (this.selectorCache[selector]);
    };
    /**
     * @template P
     * @param {?} state
     * @param {?} action
     * @return {?}
     */
    ReduxStateProvider.prototype.reduce = function (state, action) {
        var /** @type {?} */ reducerMethods = this.reducerMethodsByType[action.type] || [];
        return reducerMethods.reduce(function (stateToReduce, method) { return method(stateToReduce, action); }, state);
    };
    /**
     * @param {?} reducerClassInstance
     * @return {?}
     */
    ReduxStateProvider.prototype.getReducerMethods = function (reducerClassInstance) {
        return Object.values(Object.getPrototypeOf(reducerClassInstance))
            .map(function (method) {
            return {
                method: ((method)).bind(reducerClassInstance),
                type: ReduxReducerDecorator.get(method),
            };
        })
            .filter(function (type) { return type != null; })
            .reduce(function (all, curr) { return all.concat([].concat(curr.type).map(function (type) { return (Object.assign({}, curr, { type: type })); })); }, []);
    };
    return ReduxStateProvider;
}());
ReduxStateProvider.instancesByName = {};
/** @nocollapse */
ReduxStateProvider.ctorParameters = function () { return [
    { type: Array, decorators: [{ type: Inject, args: [StateDefToken,] },] },
]; };
/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
/**
 * @template S
 * @param {?} expression
 * @param {?=} context
 * @return {?}
 */
function ReduxSelect(expression, context) {
    return function (target, propertyKey) {
        var /** @type {?} */ originalOnInit = target['ngOnInit'];
        Object.defineProperty(target, 'ngOnInit', {
            value: function (injector) {
                var /** @type {?} */ stateName = ReduxStateDecorator.get(context).name;
                Object.defineProperty(target, propertyKey, {
                    value: ReduxStateProvider.instancesByName[stateName].select(expression)
                });
                if (originalOnInit) {
                    originalOnInit.apply(target, arguments);
                }
            }
        });
    };
}
/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
var ReduxState = ReduxStateDecorator.forClass;
/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
var ReduxStore = /** @class */ (function (_super) {
    __extends(ReduxStore, _super);
    function ReduxStore() {
        return _super.call(this, 'ReduxStore') || this;
    }
    return ReduxStore;
}(InjectionToken));
/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
var ReduxCommonModule = /** @class */ (function () {
    function ReduxCommonModule() {
    }
    return ReduxCommonModule;
}());
ReduxCommonModule.decorators = [
    { type: NgModule, args: [{
                declarations: [
                    ReduxSelectPipe,
                ],
                exports: [
                    ReduxSelectPipe,
                ],
            },] },
];
/** @nocollapse */
ReduxCommonModule.ctorParameters = function () { return []; };
/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
var ReduxTestingStore = /** @class */ (function () {
    function ReduxTestingStore() {
        this.state = new BehaviorSubject$1(null);
    }
    /**
     * @template S
     * @param {?} state
     * @param {?} value
     * @return {?}
     */
    ReduxTestingStore.prototype.setState = function (state, value) {
        var _this = this;
        var name = ReduxStateDecorator.get(state).name;
        var /** @type {?} */ nextState = Object.assign({}, this.state.getValue(), (_a = {},
            _a[name] = value,
            _a));
        this.state.next(nextState);
        return this.state
            .takeWhile(function (currentState) { return currentState !== nextState; })
            .toPromise().then(function () { return _this.state.getValue(); });
        var _a;
    };
    /**
     * @return {?}
     */
    ReduxTestingStore.prototype.getState = function () {
        return this.state.getValue();
    };
    /**
     * @param {?} listener
     * @return {?}
     */
    ReduxTestingStore.prototype.subscribe = function (listener) {
        return this.state.subscribe.call(this.state, listener);
    };
    /**
     * @return {?}
     */
    ReduxTestingStore.prototype.replaceReducer = function () {
    };
    /**
     * @template T
     * @param {?} action
     * @return {?}
     */
    ReduxTestingStore.prototype.dispatch = function (action) {
        return action;
    };
    return ReduxTestingStore;
}());
ReduxTestingStore.decorators = [
    { type: Injectable },
];
/** @nocollapse */
ReduxTestingStore.ctorParameters = function () { return []; };
/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
var ReduxTestingModule = /** @class */ (function () {
    /**
     * @param {?=} store
     */
    function ReduxTestingModule(store) {
        if (store === void 0) { store = null; }
        if (store) {
            Registry.reset();
            Registry.registerStore(store);
        }
    }
    /**
     * @param {?=} config
     * @return {?}
     */
    ReduxTestingModule.forRoot = function (config) {
        if (config === void 0) { config = {}; }
        return {
            ngModule: ReduxTestingModule,
            providers: [
                { provide: ReduxTestingStore, useFactory: config.storeFactory || ReduxTestingModule.defaultStoreFactory },
                { provide: StateDefToken, useValue: config.state || null, multi: true },
            ],
        };
    };
    /**
     * @return {?}
     */
    ReduxTestingModule.defaultStoreFactory = function () {
        return new ReduxTestingStore();
    };
    return ReduxTestingModule;
}());
ReduxTestingModule.decorators = [
    { type: NgModule, args: [{
                exports: [
                    ReduxCommonModule,
                ],
                imports: [
                    CommonModule,
                ],
                providers: [
                    ReduxTestingStore,
                ],
            },] },
];
/** @nocollapse */
ReduxTestingModule.ctorParameters = function () { return [
    { type: undefined, decorators: [{ type: Optional }, { type: Inject, args: [ReduxTestingStore,] },] },
]; };
/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
var ReducerProvider = /** @class */ (function () {
    function ReducerProvider() {
        this.stateProviders = {};
    }
    /**
     * @param {?} provider
     * @return {?}
     */
    ReducerProvider.prototype.addStateProvider = function (provider) {
        if (!this.stateProviders[provider.name]) {
            Registry.registerState(provider);
            this.stateProviders[provider.name] = provider;
        }
    };
    /**
     * @param {?} rootState
     * @param {?} action
     * @return {?}
     */
    ReducerProvider.prototype.reduce = function (rootState, action) {
        if (action.type === Registry.ACTION_REGISTER_STATE) {
            var /** @type {?} */ regAction = (((action)));
            return Object.assign({}, rootState, (_a = {}, _a[regAction.payload.name] = regAction.payload.initialValue, _a));
        }
        return Object.values(this.stateProviders).reduce(function (stateToReduce, provider) {
            return Object.assign({}, stateToReduce, (_a = {},
                _a[provider.name] = provider.reduce(stateToReduce[provider.name], action),
                _a));
            var _a;
        }, rootState);
        var _a;
    };
    return ReducerProvider;
}());
ReducerProvider.decorators = [
    { type: Injectable },
];
/** @nocollapse */
ReducerProvider.ctorParameters = function () { return []; };
/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
var ReduxRootModule = /** @class */ (function () {
    /**
     * @param {?=} store
     * @param {?=} reducer
     * @param {?=} injector
     * @param {?=} stateDefs
     */
    function ReduxRootModule(store, reducer, injector, stateDefs) {
        if (store === void 0) { store = null; }
        if (stateDefs === void 0) { stateDefs = []; }
        var _this = this;
        this.reducer = reducer;
        this.injector = injector;
        Registry.registerStore(store);
        stateDefs
            .filter(function (def) { return def && def.provider; })
            .map(function (def) { return _this.injector.get(def.provider); })
            .forEach(function (provider) { return _this.reducer.addStateProvider(provider); });
    }
    return ReduxRootModule;
}());
ReduxRootModule.decorators = [
    { type: NgModule, args: [{
                exports: [
                    ReduxCommonModule,
                ],
                imports: [
                    CommonModule,
                ],
            },] },
];
/** @nocollapse */
ReduxRootModule.ctorParameters = function () { return [
    { type: undefined, decorators: [{ type: Inject, args: [ReduxStore,] },] },
    { type: ReducerProvider, },
    { type: Injector, },
    { type: Array, decorators: [{ type: Optional }, { type: Inject, args: [StateDefToken,] },] },
]; };
var ReduxModule = /** @class */ (function () {
    /**
     * @param {?=} stateDefs
     * @param {?=} reducer
     * @param {?=} injector
     */
    function ReduxModule(stateDefs, reducer, injector) {
        if (stateDefs === void 0) { stateDefs = []; }
        var _this = this;
        this.reducer = reducer;
        this.injector = injector;
        stateDefs
            .filter(function (def) { return def && def.provider; })
            .map(function (def) { return _this.injector.get(def.provider); })
            .forEach(function (provider) { return _this.reducer.addStateProvider(provider); });
    }
    /**
     * @param {?=} config
     * @return {?}
     */
    ReduxModule.forChild = function (config) {
        if (config === void 0) { config = {}; }
        return {
            ngModule: ReduxModule,
            providers: [
                { provide: StateDefToken, useValue: config.state || null, multi: true },
            ],
        };
    };
    /**
     * @param {?=} config
     * @return {?}
     */
    ReduxModule.forRoot = function (config) {
        if (config === void 0) { config = {}; }
        return {
            ngModule: ReduxRootModule,
            providers: [
                ReducerProvider,
                {
                    provide: ReduxStore,
                    useFactory: config.storeFactory || ReduxModule.defaultStoreFactory,
                    deps: [ReducerProvider]
                },
                { provide: StateDefToken, useValue: config.state || null, multi: true },
            ],
        };
    };
    /**
     * @param {?} reducer
     * @return {?}
     */
    ReduxModule.defaultStoreFactory = function (reducer) {
        return createStore(reducer.reduce.bind(reducer), {}, ReduxModule.defaultEnhancerFactory());
    };
    /**
     * @return {?}
     */
    ReduxModule.defaultEnhancerFactory = function () {
        if (console && window['__REDUX_DEVTOOLS_EXTENSION__'] && isDevMode()) {
            return window['__REDUX_DEVTOOLS_EXTENSION__']();
        }
        return ReduxModule.noopEnhancer;
    };
    /**
     * @param {?} next
     * @return {?}
     */
    ReduxModule.noopEnhancer = function (next) {
        return next;
    };
    return ReduxModule;
}());
ReduxModule.decorators = [
    { type: NgModule, args: [{
                exports: [
                    ReduxCommonModule,
                ],
                imports: [
                    CommonModule,
                ],
            },] },
];
/** @nocollapse */
ReduxModule.ctorParameters = function () { return [
    { type: Array, decorators: [{ type: Optional }, { type: Inject, args: [StateDefToken,] },] },
    { type: ReducerProvider, },
    { type: Injector, },
]; };
/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
var ReduxAction = ReduxActionDecorator.forMethod;
var ReduxActionContext = ReduxActionContextDecorator.forClass;
var getActionType = ReduxActionDispatcher.getType;
var ReduxReducer = ReduxReducerDecorator.forMethod;
/**
 * @record
 */
/**
 * interface doesn't work \@see https://github.com/angular/angular-cli/issues/2034#issuecomment-302666897 :/
 * @deprecated Please use ActionWithPayload instead
 */
var ActionInterface = /** @class */ (function () {
    function ActionInterface() {
    }
    return ActionInterface;
}());
/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
/**
 * Generated bundle index. Do not edit.
 */
export { ReduxAction, ReduxActionContext, getActionType, ReduxReducer, ActionInterface, ReduxSelect, ReduxSelectPipe, ReduxState, ReduxStore, ReduxTestingModule, ReduxTestingStore, ReduxSelector, ReduxStateProvider, ReduxRootModule, ReduxModule, ReduxCommonModule as ɵb, ReducerProvider as ɵc, StateDefToken as ɵa };
//# sourceMappingURL=harmowatch-ngx-redux-core.js.map
